/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

package gproboproxy;

import java.io.*;
import java.util.Properties;

import ec.*;
import ec.gp.*;

import robocode.*;
import gprobocode.*;

/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author unascribed
 * @version 1.0
 */

public abstract class GPTank extends AdvancedRobot {

    // Statistics about robocode's run could be stored in Properties
    // The only safe place to store an individual is in a static var
    protected static Properties output;

    protected GPRobocodeData data;

    public abstract void setIndividual(GPIndividual ind);
    public abstract GPIndividual getIndividual();

    public Properties getOutput() {

        // Create only if needed
        if (output == null) {
            output = new Properties();
        }

        return output;
    }

    public Properties downloadOutput() {

        return output;
    }

    public void run() {

        data = new GPRobocodeData();
        data.robot = this;

        long msec = System.currentTimeMillis();

        while (true) {

            // Invoke the individual
            evaluate();

            // Clear all events
            data.events.clear();

            // Tell robocode to execute actions
            execute();

            getOutput().setProperty(
                    "LastRoundDuration",
                    String.valueOf(System.currentTimeMillis() - msec) );
        }
    }

    public void evaluate() {

        if (getIndividual() == null) {
            return;
        }

        data.resetValues();

        getIndividual().trees[0].child.eval(null, 0, data, null,
                                            getIndividual(), null);

        resolveActions();
    }

    public void uploadIndividual(byte[] array) {
        uploadIndividual(new ByteArrayInputStream(array));
    }

    public void uploadIndividual(InputStream in) {
        try {
            out.print("Uploading individual...");

            ObjectInputStream ins = new ObjectInputStream(in);

            setIndividual((GPIndividual) ins.readObject());

            ins.close();
            in.close();

            out.println("OK");
        }
        catch (Exception e) {
            out.println("FAILED");

            e.printStackTrace();
        }
    }

    public void resolveActions() {

        if (data.move > 0) {
            setAhead(data.move);
        }
        else if (data.move < 0) {
            setBack(-data.move);
        }

        if (data.turn > 0) {
            setTurnRight(data.turn);
        }
        else if (data.turn < 0) {
            setTurnLeft(-data.turn);
        }

        if (data.turnGun > 0) {
            setTurnGunRight(data.turnGun);
        }
        else if (data.turnGun < 0) {
            setTurnGunLeft(-data.turnGun);
        }

        if (data.turnRadar > 0) {
            setTurnRadarRight(data.turnRadar);
        }
        else if (data.turnRadar < 0) {
            setTurnRadarLeft(-data.turnRadar);
        }

        if (data.fire > 0) {
            setFire(data.fire);
        }
    }

    /* Event Handling */

    public void onBulletHit(BulletHitEvent event) {
        addEvent(event);
    }

    public void onBulletHitBullet(BulletHitBulletEvent event) {
        addEvent(event);
    }

    public void onBulletMissed(BulletMissedEvent event) {
        addEvent(event);
    }

    public void onHitByBullet(HitByBulletEvent event) {
        addEvent(event);
    }

    public void onHitRobot(HitRobotEvent event) {
        addEvent(event);
    }

    public void onHitWall(HitWallEvent event) {
        addEvent(event);
    }

    public void onRobotDeath(RobotDeathEvent event) {
        addEvent(event);
    }

    public void onScannedRobot(ScannedRobotEvent event) {
        addEvent(event);
    }

    public void onSkippedTurn(SkippedTurnEvent event) {
        addEvent(event);
    }

    public void onWin(WinEvent event) {
        addEvent(event);
    }

    public void addEvent(Event event) {
        data.events.addElement(event);
    }

    public void clearEvents() {
        data.events.clear();
    }

}

